@@ -448,7 +448,12 @@ impl Serialize for Metadata {
         }
 
         let tensors: Vec<_> = names.iter().zip(self.tensors.iter()).collect();
-        let mut map = serializer.serialize_map(Some(tensors.len()))?;
+        let length = if let Some(metadata) = &self.metadata {
+            metadata.len()
+        } else {
+            0
+        };
+        let mut map = serializer.serialize_map(Some(tensors.len() + length))?;
         if let Some(metadata) = &self.metadata {
             map.serialize_entry("__metadata__", metadata)?;
         }
@@ -641,6 +646,12 @@ pub enum Dtype {
     U8,
     /// Signed byte
     I8,
+    /// FP8 <https://arxiv.org/pdf/2209.05433.pdf>_
+    #[allow(non_camel_case_types)]
+    F8_E5M2,
+    /// FP8 <https://arxiv.org/pdf/2209.05433.pdf>_
+    #[allow(non_camel_case_types)]
+    F8_E4M3,
     /// Signed integer (16-bit)
     I16,
     /// Unsigned integer (16-bit)
@@ -670,6 +681,8 @@ impl Dtype {
             Dtype::BOOL => 1,
             Dtype::U8 => 1,
             Dtype::I8 => 1,
+            Dtype::F8_E5M2 => 1,
+            Dtype::F8_E4M3 => 1,
             Dtype::I16 => 2,
             Dtype::U16 => 2,
             Dtype::I32 => 4,
@@ -825,6 +838,34 @@ mod tests {
         let _parsed = SafeTensors::deserialize(&out).unwrap();
     }
 
+    #[test]
+    fn test_empty() {
+        let tensors: HashMap<String, TensorView> = HashMap::new();
+
+        let out = serialize(&tensors, &None).unwrap();
+        assert_eq!(
+            out,
+            [8, 0, 0, 0, 0, 0, 0, 0, 123, 125, 32, 32, 32, 32, 32, 32]
+        );
+        let _parsed = SafeTensors::deserialize(&out).unwrap();
+
+        let metadata: Option<HashMap<String, String>> = Some(
+            [("framework".to_string(), "pt".to_string())]
+                .into_iter()
+                .collect(),
+        );
+        let out = serialize(&tensors, &metadata).unwrap();
+        assert_eq!(
+            out,
+            [
+                40, 0, 0, 0, 0, 0, 0, 0, 123, 34, 95, 95, 109, 101, 116, 97, 100, 97, 116, 97, 95,
+                95, 34, 58, 123, 34, 102, 114, 97, 109, 101, 119, 111, 114, 107, 34, 58, 34, 112,
+                116, 34, 125, 125, 32, 32, 32, 32, 32
+            ]
+        );
+        let _parsed = SafeTensors::deserialize(&out).unwrap();
+    }
+
     #[test]
     fn test_serialization_forced_alignement() {
         let data: Vec<u8> = vec![0.0f32, 1.0, 2.0, 3.0, 4.0, 5.0]
